#import "ModelSwarm.h"
#import <simtools.h>

@implementation ModelSwarm

// createBegin: here we set up the default simulation parameters.
+createBegin: (id) aZone {
  ModelSwarm * obj;
  id <ProbeMap> probeMap;

  // First, call our superclass createBegin - the return value is the
  // allocated HeatbugModelSwarm object.
  obj = [super createBegin: aZone];

  // Now fill in various simulation parameters with default values.
  obj->numAgents = 5;

  probeMap = [EmptyProbeMap createBegin: aZone];
  [probeMap setProbedClass: [self class]];
  probeMap = [probeMap createEnd];

  // Add in a bunch of variables, one per simulation parameter
  [probeMap addProbe: [probeLibrary getProbeForVariable: "numAgents"
				    inClass: [self class]]];

  // Now install our custom probeMap into the probeLibrary.
  [probeLibrary setProbeMap: probeMap For: [self class]];
  
  return obj;
}

// createEnd: we could create some objects here if we knew we needed
// them. But this method is called before the user is allowed to fill
// in any customization of the model, so we defer most object creation
// to later. (In this example, this method does nothing at all and could
// just be inherited. But it's here to show you a place to customize.)
-createEnd {
  return [super createEnd];
}

-buildObjects {
  int i;

  [super buildObjects];

  // mapNew=[Map create: self];
  mapNew=[List create: self];
  
  for(i=0;i<numAgents;i++) 
    [mapNew addLast: [[[Holder create: self] setHolderValue: i] setHolderOffset: i]];

  /*[mapNew at:    [[[Holder create: self] setHolderValue: i] setHolderOffset: i]
    insert:[[[Holder create: self] setHolderValue: i] setHolderOffset: i]];
  */       

  return self;
}

-buildActions {
  [super buildActions];
  
  modelSchedule = [Schedule createBegin: [self getZone]];
  [modelSchedule setRepeatInterval: 1];
  modelSchedule = [modelSchedule createEnd];
  [modelSchedule at: 0 createActionTo: self message: M(newGeneration)];

  return self;
}

-activateIn: (id) swarmContext {
  // First, activate ourselves via the superclass activateIn: method.
  // Just pass along the context: the activity library does the right thing.
  [super activateIn: swarmContext];

  // Now activate our own schedule.
  [modelSchedule activateIn: self];

  // Finally, return our activity.
  return [self getSwarmActivity];
}

-newGeneration {
  id index;
  id <HolderObject> member;
  
  printf("%ld Members in mapOld:\n",[[self getActivity] getCurrentTime]);
  [mapOld forEach:M(print)];
  printf("%ld Members in mapNew:\n",[[self getActivity] getCurrentTime]);
  [mapNew forEach:M(print)];
  
  index=[mapNew begin: self];
  while((member=[index next])) {
    [member setHolderValue: [member getHolderValue]+100];
  }
  [index drop];

 
  [mapOld deleteAll]; // Remove this to make it work
  [mapOld drop];
  mapOld = [mapNew copy: self];

  printf("%ld (After copy) Members in mapOld:\n",[[self getActivity] getCurrentTime]);
  [mapOld forEach:M(print)];
  printf("%ld (After copy) Members in mapNew:\n",[[self getActivity] getCurrentTime]);
  [mapNew forEach:M(print)];


  

  return self;
}

@end





